"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.patchUpdatesByDeviceId = exports.deleteUpdatesByDeviceId = exports.getUpdatesByDeviceId = exports.postUpdates = exports.getUpdates = void 0;
const data_source_1 = require("../data_source");
const format_1 = require("../methods/format");
const write_1 = require("../methods/write");
const model_1 = require("../model");
const service_common_1 = require("@crosslab/service-common");
/**
 * This function implements the functionality for handling POST requests on /updates endpoint.
 * @param _user The user submitting the request.
 */
const getUpdates = async (_user) => {
    const updateInformationRepository = data_source_1.AppDataSource.getRepository(model_1.UpdateInformationModel);
    const updateInformationArray = await updateInformationRepository.find();
    return {
        status: 200,
        body: updateInformationArray.map(format_1.formatUpdateInformation),
    };
};
exports.getUpdates = getUpdates;
/**
 * This function implements the functionality for handling POST requests on /updates endpoint.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 */
const postUpdates = async (body, _user) => {
    const updateInformationRepository = data_source_1.AppDataSource.getRepository(model_1.UpdateInformationModel);
    const updateInformation = updateInformationRepository.create();
    (0, write_1.writeUpdateInformation)(updateInformation, body);
    await updateInformationRepository.save(updateInformation);
    return {
        status: 201,
        body: (0, format_1.formatUpdateInformation)(updateInformation),
    };
};
exports.postUpdates = postUpdates;
/**
 * This function implements the functionality for handling GET requests on /updates/{device_id} endpoint.
 * @param parameters The parameters of the request.
 */
const getUpdatesByDeviceId = async (parameters) => {
    const updateInformationRepository = data_source_1.AppDataSource.getRepository(model_1.UpdateInformationModel);
    const updateInformation = await updateInformationRepository.findOneBy({
        device_id: parameters.device_id,
    });
    if (!updateInformation) {
        throw new service_common_1.MissingEntityError(`Could not find update for device ${parameters.device_id}`, 404);
    }
    if (parameters.current_version !== updateInformation.latest_version) {
        return {
            status: 303,
            headers: {
                Location: updateInformation.latest_version_link,
            },
        };
    }
    return {
        status: 200,
    };
};
exports.getUpdatesByDeviceId = getUpdatesByDeviceId;
/**
 * This function implements the functionality for handling DELETE requests on /updates/{device_id} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 */
const deleteUpdatesByDeviceId = async (parameters, _user) => {
    const updateInformationRepository = data_source_1.AppDataSource.getRepository(model_1.UpdateInformationModel);
    const updateInformation = await updateInformationRepository.findOneBy({
        device_id: parameters.device_id,
    });
    if (!updateInformation) {
        throw new service_common_1.MissingEntityError(`Could not find update for device ${parameters.device_id}`, 404);
    }
    await updateInformationRepository.delete(updateInformation);
    return {
        status: 204,
    };
};
exports.deleteUpdatesByDeviceId = deleteUpdatesByDeviceId;
/**
 * This function implements the functionality for handling PATCH requests on /updates/{device_id} endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 */
const patchUpdatesByDeviceId = async (parameters, body, _user) => {
    const updateInformationRepository = data_source_1.AppDataSource.getRepository(model_1.UpdateInformationModel);
    const updateInformation = await updateInformationRepository.findOneBy({
        device_id: parameters.device_id,
    });
    if (!updateInformation) {
        throw new service_common_1.MissingEntityError(`Could not find update for device ${parameters.device_id}`, 404);
    }
    if (updateInformation.device_id != body.device_id) {
        throw new service_common_1.InvalidChangeError(`Cannot change device id of update`, 400);
    }
    (0, write_1.writeUpdateInformation)(updateInformation, body);
    await updateInformationRepository.save(updateInformation);
    return {
        status: 200,
        body: (0, format_1.formatUpdateInformation)(updateInformation),
    };
};
exports.patchUpdatesByDeviceId = patchUpdatesByDeviceId;
//# sourceMappingURL=updates.js.map